@@ -1,10 +1,12 @@
+# SPDX-License-Identifier: MIT
+
 """
 Tests for dunder methods from `attrib._make`.
 """
 
-from __future__ import absolute_import, division, print_function
 
 import copy
+import inspect
 import pickle
 
 import pytest
@@ -18,9 +20,7 @@ from attr._make import (
     NOTHING,
     Factory,
     _add_repr,
-    _is_slot_cls,
     _make_init,
-    _Nothing,
     fields,
     make_class,
 )
@@ -36,19 +36,44 @@ OrderCSlots = simple_class(order=True, slots=True)
 ReprC = simple_class(repr=True)
 ReprCSlots = simple_class(repr=True, slots=True)
 
+
+@attr.s(eq=True)
+class EqCallableC:
+    a = attr.ib(eq=str.lower, order=False)
+    b = attr.ib(eq=True)
+
+
+@attr.s(eq=True, slots=True)
+class EqCallableCSlots:
+    a = attr.ib(eq=str.lower, order=False)
+    b = attr.ib(eq=True)
+
+
+@attr.s(order=True)
+class OrderCallableC:
+    a = attr.ib(eq=True, order=str.lower)
+    b = attr.ib(order=True)
+
+
+@attr.s(order=True, slots=True)
+class OrderCallableCSlots:
+    a = attr.ib(eq=True, order=str.lower)
+    b = attr.ib(order=True)
+
+
 # HashC is hashable by explicit definition while HashCSlots is hashable
 # implicitly.  The "Cached" versions are the same, except with hash code
 # caching enabled
-HashC = simple_class(hash=True)
-HashCSlots = simple_class(hash=None, eq=True, frozen=True, slots=True)
-HashCCached = simple_class(hash=True, cache_hash=True)
+HashC = simple_class(unsafe_hash=True)
+HashCSlots = simple_class(unsafe_hash=None, eq=True, frozen=True, slots=True)
+HashCCached = simple_class(unsafe_hash=True, cache_hash=True)
 HashCSlotsCached = simple_class(
-    hash=None, eq=True, frozen=True, slots=True, cache_hash=True
+    unsafe_hash=None, eq=True, frozen=True, slots=True, cache_hash=True
 )
 # the cached hash code is stored slightly differently in this case
 # so it needs to be tested separately
 HashCFrozenNotSlotsCached = simple_class(
-    frozen=True, slots=False, hash=True, cache_hash=True
+    frozen=True, slots=False, unsafe_hash=True, cache_hash=True
 )
 
 
@@ -59,28 +84,37 @@ def _add_init(cls, frozen):
     This function used to be part of _make.  It wasn't used anymore however
     the tests for it are still useful to test the behavior of _make_init.
     """
+    has_pre_init = bool(getattr(cls, "__attrs_pre_init__", False))
+
     cls.__init__ = _make_init(
         cls,
         cls.__attrs_attrs__,
+        has_pre_init,
+        (
+            len(inspect.signature(cls.__attrs_pre_init__).parameters) > 1
+            if has_pre_init
+            else False
+        ),
         getattr(cls, "__attrs_post_init__", False),
         frozen,
-        _is_slot_cls(cls),
+        "__slots__" in cls.__dict__,
         cache_hash=False,
         base_attr_map={},
         is_exc=False,
-        has_global_on_setattr=False,
+        cls_on_setattr=None,
+        attrs_init=False,
     )
     return cls
 
 
-class InitC(object):
+class InitC:
     __attrs_attrs__ = [simple_attr("a"), simple_attr("b")]
 
 
 InitC = _add_init(InitC, False)
 
 
-class TestEqOrder(object):
+class TestEqOrder:
     """
     Tests for eq and order related methods.
     """
@@ -104,6 +138,16 @@ class TestEqOrder(object):
         assert cls(1, 2) == cls(1, 2)
         assert not (cls(1, 2) != cls(1, 2))
 
+    @pytest.mark.parametrize("cls", [EqCallableC, EqCallableCSlots])
+    def test_equal_callable(self, cls):
+        """
+        Equal objects are detected as equal.
+        """
+        assert cls("Test", 1) == cls("test", 1)
+        assert cls("Test", 1) != cls("test", 2)
+        assert not (cls("Test", 1) != cls("test", 1))
+        assert not (cls("Test", 1) == cls("test", 2))
+
     @pytest.mark.parametrize("cls", [EqC, EqCSlots])
     def test_unequal_same_class(self, cls):
         """
@@ -112,14 +156,24 @@ class TestEqOrder(object):
         assert cls(1, 2) != cls(2, 1)
         assert not (cls(1, 2) == cls(2, 1))
 
-    @pytest.mark.parametrize("cls", [EqC, EqCSlots])
+    @pytest.mark.parametrize("cls", [EqCallableC, EqCallableCSlots])
+    def test_unequal_same_class_callable(self, cls):
+        """
+        Unequal objects of correct type are detected as unequal.
+        """
+        assert cls("Test", 1) != cls("foo", 2)
+        assert not (cls("Test", 1) == cls("foo", 2))
+
+    @pytest.mark.parametrize(
+        "cls", [EqC, EqCSlots, EqCallableC, EqCallableCSlots]
+    )
     def test_unequal_different_class(self, cls):
         """
         Unequal objects of different type are detected even if their attributes
         match.
         """
 
-        class NotEqC(object):
+        class NotEqC:
             a = 1
             b = 2
 
@@ -138,7 +192,21 @@ class TestEqOrder(object):
         ]:
             assert cls(*a) < cls(*b)
 
-    @pytest.mark.parametrize("cls", [OrderC, OrderCSlots])
+    @pytest.mark.parametrize("cls", [OrderCallableC, OrderCallableCSlots])
+    def test_lt_callable(self, cls):
+        """
+        __lt__ compares objects as tuples of attribute values.
+        """
+        # Note: "A" < "a"
+        for a, b in [
+            (("test1", 1), ("Test1", 2)),
+            (("test0", 1), ("Test1", 1)),
+        ]:
+            assert cls(*a) < cls(*b)
+
+    @pytest.mark.parametrize(
+        "cls", [OrderC, OrderCSlots, OrderCallableC, OrderCallableCSlots]
+    )
     def test_lt_unordable(self, cls):
         """
         __lt__ returns NotImplemented if classes differ.
@@ -159,7 +227,23 @@ class TestEqOrder(object):
         ]:
             assert cls(*a) <= cls(*b)
 
-    @pytest.mark.parametrize("cls", [OrderC, OrderCSlots])
+    @pytest.mark.parametrize("cls", [OrderCallableC, OrderCallableCSlots])
+    def test_le_callable(self, cls):
+        """
+        __le__ compares objects as tuples of attribute values.
+        """
+        # Note: "A" < "a"
+        for a, b in [
+            (("test1", 1), ("Test1", 1)),
+            (("test1", 1), ("Test1", 2)),
+            (("test0", 1), ("Test1", 1)),
+            (("test0", 2), ("Test1", 1)),
+        ]:
+            assert cls(*a) <= cls(*b)
+
+    @pytest.mark.parametrize(
+        "cls", [OrderC, OrderCSlots, OrderCallableC, OrderCallableCSlots]
+    )
     def test_le_unordable(self, cls):
         """
         __le__ returns NotImplemented if classes differ.
@@ -178,7 +262,21 @@ class TestEqOrder(object):
         ]:
             assert cls(*a) > cls(*b)
 
-    @pytest.mark.parametrize("cls", [OrderC, OrderCSlots])
+    @pytest.mark.parametrize("cls", [OrderCallableC, OrderCallableCSlots])
+    def test_gt_callable(self, cls):
+        """
+        __gt__ compares objects as tuples of attribute values.
+        """
+        # Note: "A" < "a"
+        for a, b in [
+            (("Test1", 2), ("test1", 1)),
+            (("Test1", 1), ("test0", 1)),
+        ]:
+            assert cls(*a) > cls(*b)
+
+    @pytest.mark.parametrize(
+        "cls", [OrderC, OrderCSlots, OrderCallableC, OrderCallableCSlots]
+    )
     def test_gt_unordable(self, cls):
         """
         __gt__ returns NotImplemented if classes differ.
@@ -199,7 +297,23 @@ class TestEqOrder(object):
         ]:
             assert cls(*a) >= cls(*b)
 
-    @pytest.mark.parametrize("cls", [OrderC, OrderCSlots])
+    @pytest.mark.parametrize("cls", [OrderCallableC, OrderCallableCSlots])
+    def test_ge_callable(self, cls):
+        """
+        __ge__ compares objects as tuples of attribute values.
+        """
+        # Note: "A" < "a"
+        for a, b in [
+            (("Test1", 1), ("test1", 1)),
+            (("Test1", 2), ("test1", 1)),
+            (("Test1", 1), ("test0", 1)),
+            (("Test1", 1), ("test0", 2)),
+        ]:
+            assert cls(*a) >= cls(*b)
+
+    @pytest.mark.parametrize(
+        "cls", [OrderC, OrderCSlots, OrderCallableC, OrderCallableCSlots]
+    )
     def test_ge_unordable(self, cls):
         """
         __ge__ returns NotImplemented if classes differ.
@@ -207,12 +321,11 @@ class TestEqOrder(object):
         assert NotImplemented == (cls(1, 2).__ge__(42))
 
 
-class TestAddRepr(object):
+class TestAddRepr:
     """
     Tests for `_add_repr`.
     """
 
-    @pytest.mark.parametrize("slots", [True, False])
     def test_repr(self, slots):
         """
         If `repr` is False, ignore that attribute.
@@ -240,7 +353,7 @@ class TestAddRepr(object):
             return "foo:" + str(value)
 
         @attr.s
-        class C(object):
+        class C:
             a = attr.ib(repr=custom_repr)
 
         assert "C(a=foo:1)" == repr(C(1))
@@ -252,7 +365,7 @@ class TestAddRepr(object):
         """
 
         @attr.s
-        class Cycle(object):
+        class Cycle:
             value = attr.ib(default=7)
             cycle = attr.ib(default=None)
 
@@ -260,12 +373,29 @@ class TestAddRepr(object):
         cycle.cycle = cycle
         assert "Cycle(value=7, cycle=...)" == repr(cycle)
 
+    def test_infinite_recursion_long_cycle(self):
+        """
+        A cyclic graph can pass through other non-attrs objects, and repr will
+        still emit an ellipsis and not raise an exception.
+        """
+
+        @attr.s
+        class LongCycle:
+            value = attr.ib(default=14)
+            cycle = attr.ib(default=None)
+
+        cycle = LongCycle()
+        # Ensure that the reference cycle passes through a non-attrs object.
+        # This demonstrates the need for a thread-local "global" ID tracker.
+        cycle.cycle = {"cycle": [cycle]}
+        assert "LongCycle(value=14, cycle={'cycle': [...]})" == repr(cycle)
+
     def test_underscores(self):
         """
         repr does not strip underscores.
         """
 
-        class C(object):
+        class C:
             __attrs_attrs__ = [simple_attr("_x")]
 
         C = _add_repr(C)
@@ -313,22 +443,22 @@ class TestAddRepr(object):
 
 # these are for use in TestAddHash.test_cache_hash_serialization
 # they need to be out here so they can be un-pickled
-@attr.attrs(hash=True, cache_hash=False)
-class HashCacheSerializationTestUncached(object):
+@attr.attrs(unsafe_hash=True, cache_hash=False)
+class HashCacheSerializationTestUncached:
     foo_value = attr.ib()
 
 
-@attr.attrs(hash=True, cache_hash=True)
-class HashCacheSerializationTestCached(object):
+@attr.attrs(unsafe_hash=True, cache_hash=True)
+class HashCacheSerializationTestCached:
     foo_value = attr.ib()
 
 
-@attr.attrs(slots=True, hash=True, cache_hash=True)
-class HashCacheSerializationTestCachedSlots(object):
+@attr.attrs(slots=True, unsafe_hash=True, cache_hash=True)
+class HashCacheSerializationTestCachedSlots:
     foo_value = attr.ib()
 
 
-class IncrementingHasher(object):
+class IncrementingHasher:
     def __init__(self):
         self.hash_value = 100
 
@@ -338,7 +468,7 @@ class IncrementingHasher(object):
         return rv
 
 
-class TestAddHash(object):
+class TestAddHash:
     """
     Tests for `_add_hash`.
     """
@@ -351,7 +481,7 @@ class TestAddHash(object):
         exc_args = ("Invalid value for hash.  Must be True, False, or None.",)
 
         with pytest.raises(TypeError) as e:
-            make_class("C", {}, hash=1),
+            make_class("C", {}, unsafe_hash=1),
 
         assert exc_args == e.value.args
 
@@ -371,13 +501,18 @@ class TestAddHash(object):
             "enabled.",
         )
         with pytest.raises(TypeError) as e:
-            make_class("C", {}, hash=False, cache_hash=True)
+            make_class("C", {}, unsafe_hash=False, cache_hash=True)
         assert exc_args == e.value.args
 
         # unhashable case
         with pytest.raises(TypeError) as e:
             make_class(
-                "C", {}, hash=None, eq=True, frozen=False, cache_hash=True
+                "C",
+                {},
+                unsafe_hash=None,
+                eq=True,
+                frozen=False,
+                cache_hash=True,
             )
         assert exc_args == e.value.args
 
@@ -391,7 +526,7 @@ class TestAddHash(object):
             " init must be True.",
         )
         with pytest.raises(TypeError) as e:
-            make_class("C", {}, init=False, hash=True, cache_hash=True)
+            make_class("C", {}, init=False, unsafe_hash=True, cache_hash=True)
         assert exc_args == e.value.args
 
     @given(booleans(), booleans())
@@ -403,7 +538,7 @@ class TestAddHash(object):
             "C",
             {"a": attr.ib(hash=False), "b": attr.ib()},
             slots=slots,
-            hash=True,
+            unsafe_hash=True,
             cache_hash=cache_hash,
         )
 
@@ -499,13 +634,13 @@ class TestAddHash(object):
         Uncached = make_class(
             "Uncached",
             {"hash_counter": attr.ib(factory=HashCounter)},
-            hash=True,
+            unsafe_hash=True,
             cache_hash=False,
         )
         Cached = make_class(
             "Cached",
             {"hash_counter": attr.ib(factory=HashCounter)},
-            hash=True,
+            unsafe_hash=True,
             cache_hash=True,
         )
 
@@ -521,21 +656,19 @@ class TestAddHash(object):
         assert 1 == cached_instance.hash_counter.times_hash_called
 
     @pytest.mark.parametrize("cache_hash", [True, False])
-    @pytest.mark.parametrize("frozen", [True, False])
-    @pytest.mark.parametrize("slots", [True, False])
     def test_copy_hash_cleared(self, cache_hash, frozen, slots):
         """
         Test that the default hash is recalculated after a copy operation.
         """
 
-        kwargs = dict(frozen=frozen, slots=slots, cache_hash=cache_hash,)
+        kwargs = {"frozen": frozen, "slots": slots, "cache_hash": cache_hash}
 
         # Give it an explicit hash if we don't have an implicit one
         if not frozen:
-            kwargs["hash"] = True
+            kwargs["unsafe_hash"] = True
 
         @attr.s(**kwargs)
-        class C(object):
+        class C:
             x = attr.ib()
 
         a = C(IncrementingHasher())
@@ -550,7 +683,7 @@ class TestAddHash(object):
         assert orig_hash != hash(b)
 
     @pytest.mark.parametrize(
-        "klass,cached",
+        ("klass", "cached"),
         [
             (HashCacheSerializationTestUncached, False),
             (HashCacheSerializationTestCached, True),
@@ -576,7 +709,6 @@ class TestAddHash(object):
 
         assert original_hash != hash(obj_rt)
 
-    @pytest.mark.parametrize("frozen", [True, False])
     def test_copy_two_arg_reduce(self, frozen):
         """
         If __getstate__ returns None, the tuple returned by object.__reduce__
@@ -584,8 +716,8 @@ class TestAddHash(object):
         __reduce__ generated when cache_hash=True works in that case.
         """
 
-        @attr.s(frozen=frozen, cache_hash=True, hash=True)
-        class C(object):
+        @attr.s(frozen=frozen, cache_hash=True, unsafe_hash=True)
+        class C:
             x = attr.ib()
 
             def __getstate__(self):
@@ -603,7 +735,7 @@ class TestAddHash(object):
         return pickle.loads(pickle_str)
 
 
-class TestAddInit(object):
+class TestAddInit:
     """
     Tests for `_add_init`.
     """
@@ -622,9 +754,8 @@ class TestAddInit(object):
         with pytest.raises(TypeError) as e:
             C(a=1, b=2)
 
-        assert (
+        assert e.value.args[0].endswith(
             "__init__() got an unexpected keyword argument 'a'"
-            == e.value.args[0]
         )
 
     @given(booleans(), booleans())
@@ -677,7 +808,7 @@ class TestAddInit(object):
         If a default value is present, it's used as fallback.
         """
 
-        class C(object):
+        class C:
             __attrs_attrs__ = [
                 simple_attr(name="a", default=2),
                 simple_attr(name="b", default="hallo"),
@@ -695,10 +826,10 @@ class TestAddInit(object):
         If a default factory is present, it's used as fallback.
         """
 
-        class D(object):
+        class D:
             pass
 
-        class C(object):
+        class C:
             __attrs_attrs__ = [
                 simple_attr(name="a", default=Factory(list)),
                 simple_attr(name="b", default=Factory(D)),
@@ -710,6 +841,29 @@ class TestAddInit(object):
         assert [] == i.a
         assert isinstance(i.b, D)
 
+    def test_factory_takes_self(self):
+        """
+        If takes_self on factories is True, self is passed.
+        """
+        C = make_class(
+            "C",
+            {
+                "x": attr.ib(
+                    default=Factory((lambda self: self), takes_self=True)
+                )
+            },
+        )
+
+        i = C()
+
+        assert i is i.x
+
+    def test_factory_hashable(self):
+        """
+        Factory is hashable.
+        """
+        assert hash(Factory(None, False)) == hash(Factory(None, False))
+
     def test_validator(self):
         """
         If a validator is passed, call it with the preliminary instance, the
@@ -773,7 +927,7 @@ class TestAddInit(object):
         underscores.
         """
 
-        class C(object):
+        class C:
             __attrs_attrs__ = [simple_attr("_private")]
 
         C = _add_init(C, False)
@@ -781,36 +935,43 @@ class TestAddInit(object):
         assert 42 == i._private
 
 
-class TestNothing(object):
+class TestNothing:
     """
-    Tests for `_Nothing`.
+    Tests for `NOTHING`.
     """
 
     def test_copy(self):
         """
         __copy__ returns the same object.
         """
-        n = _Nothing()
+        n = NOTHING
         assert n is copy.copy(n)
 
     def test_deepcopy(self):
         """
         __deepcopy__ returns the same object.
         """
-        n = _Nothing()
+        n = NOTHING
         assert n is copy.deepcopy(n)
 
     def test_eq(self):
         """
         All instances are equal.
         """
-        assert _Nothing() == _Nothing() == NOTHING
-        assert not (_Nothing() != _Nothing())
-        assert 1 != _Nothing()
+        assert NOTHING == NOTHING == NOTHING
+        assert not (NOTHING != NOTHING)
+        assert 1 != NOTHING
+
+    def test_false(self):
+        """
+        NOTHING evaluates as falsey.
+        """
+        assert not NOTHING
+        assert False is bool(NOTHING)
 
 
-@attr.s(hash=True, order=True)
-class C(object):
+@attr.s(unsafe_hash=True, order=True)
+class C:
     pass
 
 
@@ -818,12 +979,24 @@ class C(object):
 OriginalC = C
 
 
-@attr.s(hash=True, order=True)
-class C(object):
+@attr.s(unsafe_hash=True, order=True)
+class C:
     pass
 
 
-class TestFilenames(object):
+CopyC = C
+
+
+@attr.s(unsafe_hash=True, order=True)
+class C:
+    """
+    A different class, to generate different methods.
+    """
+
+    a = attr.ib()
+
+
+class TestFilenames:
     def test_filenames(self):
         """
         The created dunder methods have a "consistent" filename.
@@ -840,15 +1013,27 @@ class TestFilenames(object):
             OriginalC.__hash__.__code__.co_filename
             == "<attrs generated hash tests.test_dunders.C>"
         )
+        assert (
+            CopyC.__init__.__code__.co_filename
+            == "<attrs generated init tests.test_dunders.C>"
+        )
+        assert (
+            CopyC.__eq__.__code__.co_filename
+            == "<attrs generated eq tests.test_dunders.C>"
+        )
+        assert (
+            CopyC.__hash__.__code__.co_filename
+            == "<attrs generated hash tests.test_dunders.C>"
+        )
         assert (
             C.__init__.__code__.co_filename
-            == "<attrs generated init tests.test_dunders.C-2>"
+            == "<attrs generated init tests.test_dunders.C-1>"
         )
         assert (
             C.__eq__.__code__.co_filename
-            == "<attrs generated eq tests.test_dunders.C-2>"
+            == "<attrs generated eq tests.test_dunders.C-1>"
         )
         assert (
             C.__hash__.__code__.co_filename
-            == "<attrs generated hash tests.test_dunders.C-2>"
+            == "<attrs generated hash tests.test_dunders.C-1>"
         )
