@@ -1,36 +1,33 @@
+# SPDX-License-Identifier: MIT
+
 """
 End-to-end tests.
 """
 
-from __future__ import absolute_import, division, print_function
-
+import inspect
 import pickle
 
 from copy import deepcopy
 
 import pytest
-import six
 
-from hypothesis import assume, given
+from hypothesis import given
 from hypothesis.strategies import booleans
 
 import attr
 
-from attr._compat import PY2, TYPE
 from attr._make import NOTHING, Attribute
 from attr.exceptions import FrozenInstanceError
 
-from .strategies import optional_bool
-
 
 @attr.s
-class C1(object):
+class C1:
     x = attr.ib(validator=attr.validators.instance_of(int))
     y = attr.ib()
 
 
 @attr.s(slots=True)
-class C1Slots(object):
+class C1Slots:
     x = attr.ib(validator=attr.validators.instance_of(int))
     y = attr.ib()
 
@@ -39,19 +36,19 @@ foo = None
 
 
 @attr.s()
-class C2(object):
+class C2:
     x = attr.ib(default=foo)
     y = attr.ib(default=attr.Factory(list))
 
 
 @attr.s(slots=True)
-class C2Slots(object):
+class C2Slots:
     x = attr.ib(default=foo)
     y = attr.ib(default=attr.Factory(list))
 
 
 @attr.s
-class Base(object):
+class Base:
     x = attr.ib()
 
     def meth(self):
@@ -59,7 +56,7 @@ class Base(object):
 
 
 @attr.s(slots=True)
-class BaseSlots(object):
+class BaseSlots:
     x = attr.ib()
 
     def meth(self):
@@ -77,7 +74,7 @@ class SubSlots(BaseSlots):
 
 
 @attr.s(frozen=True, slots=True)
-class Frozen(object):
+class Frozen:
     x = attr.ib()
 
 
@@ -87,7 +84,7 @@ class SubFrozen(Frozen):
 
 
 @attr.s(frozen=True, slots=False)
-class FrozenNoSlots(object):
+class FrozenNoSlots:
     x = attr.ib()
 
 
@@ -96,21 +93,19 @@ class Meta(type):
 
 
 @attr.s
-@six.add_metaclass(Meta)
-class WithMeta(object):
+class WithMeta(metaclass=Meta):
     pass
 
 
 @attr.s(slots=True)
-@six.add_metaclass(Meta)
-class WithMetaSlots(object):
+class WithMetaSlots(metaclass=Meta):
     pass
 
 
 FromMakeClass = attr.make_class("FromMakeClass", ["x"])
 
 
-class TestFunctional(object):
+class TestFunctional:
     """
     Functional tests.
     """
@@ -123,6 +118,7 @@ class TestFunctional(object):
         assert (
             Attribute(
                 name="x",
+                alias="x",
                 default=foo,
                 validator=None,
                 repr=True,
@@ -135,6 +131,7 @@ class TestFunctional(object):
             ),
             Attribute(
                 name="y",
+                alias="y",
                 default=attr.Factory(list),
                 validator=None,
                 repr=True,
@@ -164,8 +161,7 @@ class TestFunctional(object):
 
         # Using C1 explicitly, since slotted classes don't support this.
         assert (
-            "'x' must be <{type} 'int'> (got '1' that is a <{type} "
-            "'str'>).".format(type=TYPE),
+            "'x' must be <class 'int'> (got '1' that is a <class 'str'>).",
             attr.fields(C1).x,
             int,
             "1",
@@ -178,7 +174,7 @@ class TestFunctional(object):
         """
 
         @attr.s(slots=slots)
-        class C3(object):
+        class C3:
             _x = attr.ib()
 
         assert "C3(_x=1)" == repr(C3(x=1))
@@ -193,6 +189,7 @@ class TestFunctional(object):
         assert (
             Attribute(
                 name="a",
+                alias="a",
                 default=NOTHING,
                 validator=None,
                 repr=True,
@@ -205,6 +202,7 @@ class TestFunctional(object):
             ),
             Attribute(
                 name="b",
+                alias="b",
                 default=NOTHING,
                 validator=None,
                 repr=True,
@@ -228,9 +226,9 @@ class TestFunctional(object):
         assert i.x is i.meth() is obj
         assert i.y == 2
         if cls is Sub:
-            assert "Sub(x={obj}, y=2)".format(obj=obj) == repr(i)
+            assert f"Sub(x={obj}, y=2)" == repr(i)
         else:
-            assert "SubSlots(x={obj}, y=2)".format(obj=obj) == repr(i)
+            assert f"SubSlots(x={obj}, y=2)" == repr(i)
 
     @pytest.mark.parametrize("base", [Base, BaseSlots])
     def test_subclass_without_extra_attrs(self, base):
@@ -245,7 +243,7 @@ class TestFunctional(object):
         obj = object()
         i = Sub2(x=obj)
         assert i.x is i.meth() is obj
-        assert "Sub2(x={obj})".format(obj=obj) == repr(i)
+        assert f"Sub2(x={obj})" == repr(i)
 
     @pytest.mark.parametrize(
         "frozen_class",
@@ -314,10 +312,7 @@ class TestFunctional(object):
         """
         Pickle object serialization works on all kinds of attrs classes.
         """
-        if len(attr.fields(cls)) == 2:
-            obj = cls(123, 456)
-        else:
-            obj = cls(123)
+        obj = cls(123, 456) if len(attr.fields(cls)) == 2 else cls(123)
 
         assert repr(obj) == repr(pickle.loads(pickle.dumps(obj, protocol)))
 
@@ -339,7 +334,7 @@ class TestFunctional(object):
         """
         Metaclass data is preserved.
         """
-        assert Meta == type(cls)
+        assert Meta is type(cls)
 
     def test_default_decorator(self):
         """
@@ -348,7 +343,7 @@ class TestFunctional(object):
         """
 
         @attr.s
-        class C(object):
+        class C:
             x = attr.ib(default=1)
             y = attr.ib()
 
@@ -358,8 +353,6 @@ class TestFunctional(object):
 
         assert C(1, 2) == C()
 
-    @pytest.mark.parametrize("slots", [True, False])
-    @pytest.mark.parametrize("frozen", [True, False])
     @pytest.mark.parametrize("weakref_slot", [True, False])
     def test_attrib_overwrite(self, slots, frozen, weakref_slot):
         """
@@ -377,7 +370,7 @@ class TestFunctional(object):
         dict-classes are never replaced.
         """
 
-        class C(object):
+        class C:
             x = attr.ib()
 
         C_new = attr.s(C)
@@ -386,27 +379,27 @@ class TestFunctional(object):
 
     def test_hash_by_id(self):
         """
-        With dict classes, hashing by ID is active for hash=False even on
-        Python 3.  This is incorrect behavior but we have to retain it for
-        backward compatibility.
+        With dict classes, hashing by ID is active for hash=False.  This is
+        incorrect behavior but we have to retain it for
+        backwards-compatibility.
         """
 
-        @attr.s(hash=False)
-        class HashByIDBackwardCompat(object):
+        @attr.s(unsafe_hash=False)
+        class HashByIDBackwardCompat:
             x = attr.ib()
 
         assert hash(HashByIDBackwardCompat(1)) != hash(
             HashByIDBackwardCompat(1)
         )
 
-        @attr.s(hash=False, eq=False)
-        class HashByID(object):
+        @attr.s(unsafe_hash=False, eq=False)
+        class HashByID:
             x = attr.ib()
 
         assert hash(HashByID(1)) != hash(HashByID(1))
 
-        @attr.s(hash=True)
-        class HashByValues(object):
+        @attr.s(unsafe_hash=True)
+        class HashByValues:
             x = attr.ib()
 
         assert hash(HashByValues(1)) == hash(HashByValues(1))
@@ -417,40 +410,49 @@ class TestFunctional(object):
         """
 
         @attr.s
-        class Unhashable(object):
+        class Unhashable:
             pass
 
         @attr.s
-        class C(object):
+        class C:
             x = attr.ib(default=Unhashable())
 
         @attr.s
         class D(C):
             pass
 
-    @pytest.mark.parametrize("slots", [True, False])
-    def test_hash_false_eq_false(self, slots):
+    def test_unsafe_hash_false_eq_false(self, slots):
         """
-        hash=False and eq=False make a class hashable by ID.
+        unsafe_hash=False and eq=False make a class hashable by ID.
         """
 
-        @attr.s(hash=False, eq=False, slots=slots)
-        class C(object):
+        @attr.s(unsafe_hash=False, eq=False, slots=slots)
+        class C:
             pass
 
         assert hash(C()) != hash(C())
 
-    @pytest.mark.parametrize("slots", [True, False])
+    def test_hash_deprecated(self):
+        """
+        Using the hash argument is deprecated.
+        """
+
     def test_eq_false(self, slots):
         """
         eq=False makes a class hashable by ID.
         """
 
         @attr.s(eq=False, slots=slots)
-        class C(object):
+        class C:
             pass
 
-        assert hash(C()) != hash(C())
+        # Ensure both objects live long enough such that their ids/hashes
+        # can't be recycled. Thanks to Ask Hjorth Larsen for pointing that
+        # out.
+        c1 = C()
+        c2 = C()
+
+        assert hash(c1) != hash(c2)
 
     def test_overwrite_base(self):
         """
@@ -459,7 +461,7 @@ class TestFunctional(object):
         """
 
         @attr.s
-        class C(object):
+        class C:
             c = attr.ib(default=100)
             x = attr.ib(default=1)
             b = attr.ib(default=23)
@@ -506,7 +508,7 @@ class TestFunctional(object):
             slots=base_slots,
             weakref_slot=base_weakref_slot,
         )
-        class Base(object):
+        class Base:
             a = attr.ib(converter=int if base_converter else None)
 
         @attr.s(
@@ -533,7 +535,7 @@ class TestFunctional(object):
         """
 
         @attr.s
-        class C(object):
+        class C:
             property = attr.ib()
             itemgetter = attr.ib()
             x = attr.ib()
@@ -542,8 +544,6 @@ class TestFunctional(object):
         assert "itemgetter" == attr.fields(C).itemgetter.name
         assert "x" == attr.fields(C).x.name
 
-    @pytest.mark.parametrize("slots", [True, False])
-    @pytest.mark.parametrize("frozen", [True, False])
     def test_auto_exc(self, slots, frozen):
         """
         Classes with auto_exc=True have a Exception-style __str__, compare and
@@ -598,8 +598,6 @@ class TestFunctional(object):
                 deepcopy(e1)
                 deepcopy(e2)
 
-    @pytest.mark.parametrize("slots", [True, False])
-    @pytest.mark.parametrize("frozen", [True, False])
     def test_auto_exc_one_attrib(self, slots, frozen):
         """
         Having one attribute works with auto_exc=True.
@@ -613,75 +611,158 @@ class TestFunctional(object):
 
         FooError(1)
 
-    @pytest.mark.parametrize("slots", [True, False])
-    @pytest.mark.parametrize("frozen", [True, False])
     def test_eq_only(self, slots, frozen):
         """
         Classes with order=False cannot be ordered.
-
-        Python 3 throws a TypeError, in Python2 we have to check for the
-        absence.
         """
 
         @attr.s(eq=True, order=False, slots=slots, frozen=frozen)
-        class C(object):
+        class C:
             x = attr.ib()
 
-        if not PY2:
-            possible_errors = (
-                "unorderable types: C() < C()",
-                "'<' not supported between instances of 'C' and 'C'",
-                "unorderable types: C < C",  # old PyPy 3
-            )
-
-            with pytest.raises(TypeError) as ei:
-                C(5) < C(6)
-
-            assert ei.value.args[0] in possible_errors
-        else:
-            i = C(42)
-            for m in ("lt", "le", "gt", "ge"):
-                assert None is getattr(i, "__%s__" % (m,), None)
-
-    @given(cmp=optional_bool, eq=optional_bool, order=optional_bool)
-    def test_cmp_deprecated_attribute(self, cmp, eq, order):
-        """
-        Accessing Attribute.cmp raises a deprecation warning but returns True
-        if cmp is True, or eq and order are *both* effectively True.
-        """
-        # These cases are invalid and raise a ValueError.
-        assume(cmp is None or (eq is None and order is None))
-        assume(not (eq is False and order is True))
-
-        if cmp is not None:
-            rv = cmp
-        elif eq is True or eq is None:
-            rv = order is None or order is True
-        elif cmp is None and eq is None and order is None:
-            rv = True
-        elif cmp is None or eq is None:
-            rv = False
-        else:
-            pytest.fail(
-                "Unexpected state: cmp=%r eq=%r order=%r" % (cmp, eq, order)
-            )
-
-        with pytest.deprecated_call() as dc:
-
-            @attr.s
-            class C(object):
-                x = attr.ib(cmp=cmp, eq=eq, order=order)
-
-            assert rv == attr.fields(C).x.cmp
-
-        if cmp is not None:
-            # Remove warning from creating the attribute if cmp is not None.
-            dc.pop()
-
-        (w,) = dc.list
-
-        assert (
-            "The usage of `cmp` is deprecated and will be removed on or after "
-            "2021-06-01.  Please use `eq` and `order` instead."
-            == w.message.args[0]
+        possible_errors = (
+            "unorderable types: C() < C()",
+            "'<' not supported between instances of 'C' and 'C'",
+            "unorderable types: C < C",  # old PyPy 3
         )
+
+        with pytest.raises(TypeError) as ei:
+            C(5) < C(6)
+
+        assert ei.value.args[0] in possible_errors
+
+    @pytest.mark.parametrize("cmp", [True, False])
+    def test_attrib_cmp_shortcut(self, slots, cmp):
+        """
+        Setting cmp on `attr.ib`s sets both eq and order.
+        """
+
+        @attr.s(slots=slots)
+        class C:
+            x = attr.ib(cmp=cmp)
+
+        assert cmp is attr.fields(C).x.eq
+        assert cmp is attr.fields(C).x.order
+
+    def test_no_setattr_if_validate_without_validators(self, slots):
+        """
+        If a class has on_setattr=attr.setters.validate (former default in NG
+        APIs) but sets no validators, don't use the (slower) setattr in
+        __init__.
+
+        Regression test for #816.
+        """
+
+        @attr.s(on_setattr=attr.setters.validate, slots=slots)
+        class C:
+            x = attr.ib()
+
+        @attr.s(on_setattr=attr.setters.validate, slots=slots)
+        class D(C):
+            y = attr.ib()
+
+        src = inspect.getsource(D.__init__)
+
+        assert "setattr" not in src
+        assert "self.x = x" in src
+        assert "self.y = y" in src
+        assert object.__setattr__ == D.__setattr__
+
+    def test_no_setattr_if_convert_without_converters(self, slots):
+        """
+        If a class has on_setattr=attr.setters.convert but sets no validators,
+        don't use the (slower) setattr in __init__.
+        """
+
+        @attr.s(on_setattr=attr.setters.convert, slots=slots)
+        class C:
+            x = attr.ib()
+
+        @attr.s(on_setattr=attr.setters.convert, slots=slots)
+        class D(C):
+            y = attr.ib()
+
+        src = inspect.getsource(D.__init__)
+
+        assert "setattr" not in src
+        assert "self.x = x" in src
+        assert "self.y = y" in src
+        assert object.__setattr__ == D.__setattr__
+
+    def test_no_setattr_with_ng_defaults(self, slots):
+        """
+        If a class has the NG default on_setattr=[convert, validate] but sets
+        no validators or converters, don't use the (slower) setattr in
+        __init__.
+        """
+
+        @attr.define(slots=slots)
+        class C:
+            x = attr.ib()
+
+        src = inspect.getsource(C.__init__)
+
+        assert "setattr" not in src
+        assert "self.x = x" in src
+        assert object.__setattr__ == C.__setattr__
+
+        @attr.define(slots=slots)
+        class D(C):
+            y = attr.ib()
+
+        src = inspect.getsource(D.__init__)
+
+        assert "setattr" not in src
+        assert "self.x = x" in src
+        assert "self.y = y" in src
+        assert object.__setattr__ == D.__setattr__
+
+    def test_on_setattr_detect_inherited_validators(self):
+        """
+        _make_init detects the presence of a validator even if the field is
+        inherited.
+        """
+
+        @attr.s(on_setattr=attr.setters.validate)
+        class C:
+            x = attr.ib(validator=42)
+
+        @attr.s(on_setattr=attr.setters.validate)
+        class D(C):
+            y = attr.ib()
+
+        src = inspect.getsource(D.__init__)
+
+        assert "_setattr = _cached_setattr_get(self)" in src
+        assert "_setattr('x', x)" in src
+        assert "_setattr('y', y)" in src
+        assert object.__setattr__ != D.__setattr__
+
+    def test_unsafe_hash(self, slots):
+        """
+        attr.s(unsafe_hash=True) makes a class hashable.
+        """
+
+        @attr.s(slots=slots, unsafe_hash=True)
+        class Hashable:
+            pass
+
+        assert hash(Hashable())
+
+    def test_init_subclass(self, slots):
+        """
+        __attrs_init_subclass__ is called on subclasses.
+        """
+        REGISTRY = []
+
+        @attr.s(slots=slots)
+        class Base:
+            @classmethod
+            def __attrs_init_subclass__(cls):
+                REGISTRY.append(cls)
+
+        @attr.s(slots=slots)
+        class ToRegister(Base):
+            pass
+
+        assert [ToRegister] == REGISTRY
