@@ -30,28 +30,15 @@ from attr.validators import (
     min_len,
     not_,
     optional,
-    provides,
+    or_,
 )
 
 from .utils import simple_attr
 
 
-@pytest.fixture(scope="module")
-def zope_interface():
-    """Provides ``zope.interface`` if available, skipping the test if not."""
-    try:
-        import zope.interface
-    except ImportError:
-        raise pytest.skip(
-            "zope-related tests skipped when zope.interface is not installed"
-        )
-
-    return zope.interface
-
-
 class TestDisableValidators:
     @pytest.fixture(autouse=True)
-    def reset_default(self):
+    def _reset_default(self):
         """
         Make sure validators are always enabled after a test.
         """
@@ -64,7 +51,9 @@ class TestDisableValidators:
         """
         assert _config._run_validators is True
 
-    @pytest.mark.parametrize("value, expected", [(True, False), (False, True)])
+    @pytest.mark.parametrize(
+        ("value", "expected"), [(True, False), (False, True)]
+    )
     def test_set_validators_disabled(self, value, expected):
         """
         Sets `_run_validators`.
@@ -73,7 +62,9 @@ class TestDisableValidators:
 
         assert _config._run_validators is expected
 
-    @pytest.mark.parametrize("value, expected", [(True, False), (False, True)])
+    @pytest.mark.parametrize(
+        ("value", "expected"), [(True, False), (False, True)]
+    )
     def test_disabled(self, value, expected):
         """
         Returns `_run_validators`.
@@ -100,11 +91,10 @@ class TestDisableValidators:
         """
         assert _config._run_validators is True
 
-        with pytest.raises(ValueError):
-            with validator_module.disabled():
-                assert _config._run_validators is False
+        with pytest.raises(ValueError), validator_module.disabled():
+            assert _config._run_validators is False
 
-                raise ValueError("haha!")
+            raise ValueError("haha!")
 
         assert _config._run_validators is True
 
@@ -311,85 +301,6 @@ class TestAnd:
         assert C.__attrs_attrs__[0].validator == C.__attrs_attrs__[1].validator
 
 
-@pytest.fixture(scope="module")
-def ifoo(zope_interface):
-    """Provides a test ``zope.interface.Interface`` in ``zope`` tests."""
-
-    class IFoo(zope_interface.Interface):
-        """
-        An interface.
-        """
-
-        def f():
-            """
-            A function called f.
-            """
-
-    return IFoo
-
-
-class TestProvides:
-    """
-    Tests for `provides`.
-    """
-
-    def test_in_all(self):
-        """
-        Verify that this validator is in ``__all__``.
-        """
-        assert provides.__name__ in validator_module.__all__
-
-    def test_success(self, zope_interface, ifoo):
-        """
-        Nothing happens if value provides requested interface.
-        """
-
-        @zope_interface.implementer(ifoo)
-        class C:
-            def f(self):
-                pass
-
-        with pytest.deprecated_call():
-            v = provides(ifoo)
-
-        v(None, simple_attr("x"), C())
-
-    def test_fail(self, ifoo):
-        """
-        Raises `TypeError` if interfaces isn't provided by value.
-        """
-        value = object()
-        a = simple_attr("x")
-
-        with pytest.deprecated_call():
-            v = provides(ifoo)
-
-        with pytest.raises(TypeError) as e:
-            v(None, a, value)
-
-        assert (
-            "'x' must provide {interface!r} which {value!r} doesn't.".format(
-                interface=ifoo, value=value
-            ),
-            a,
-            ifoo,
-            value,
-        ) == e.value.args
-
-    def test_repr(self, ifoo):
-        """
-        Returned validator has a useful `__repr__`.
-        """
-        with pytest.deprecated_call():
-            v = provides(ifoo)
-
-        assert (
-            "<provides validator for interface {interface!r}>".format(
-                interface=ifoo
-            )
-        ) == repr(v)
-
-
 @pytest.mark.parametrize(
     "validator",
     [
@@ -446,14 +357,14 @@ class TestOptional:
 
         if isinstance(validator, list):
             repr_s = (
-                "<optional validator for _AndValidator(_validators=[{func}, "
+                f"<optional validator for _AndValidator(_validators=[{always_pass!r}, "
                 "<instance_of validator for type <class 'int'>>]) or None>"
-            ).format(func=repr(always_pass))
+            )
         elif isinstance(validator, tuple):
             repr_s = (
-                "<optional validator for _AndValidator(_validators=({func}, "
+                f"<optional validator for _AndValidator(_validators=({always_pass!r}, "
                 "<instance_of validator for type <class 'int'>>)) or None>"
-            ).format(func=repr(always_pass))
+            )
         else:
             repr_s = (
                 "<optional validator for <instance_of validator for type "
@@ -480,6 +391,7 @@ class TestIn_:
         """
         v = in_([1, 2, 3])
         a = simple_attr("test")
+
         v(1, a, 3)
 
     def test_fail(self):
@@ -522,6 +434,21 @@ class TestIn_:
         v = in_([3, 4, 5])
         assert ("<in_ validator with options [3, 4, 5]>") == repr(v)
 
+    def test_is_hashable(self):
+        """
+        `in_` is hashable, so fields using it can be used with the include and
+        exclude filters.
+        """
+
+        @attr.s
+        class C:
+            x: int = attr.ib(validator=attr.validators.in_({1, 2}))
+
+        i = C(2)
+
+        attr.asdict(i, filter=attr.filters.include(lambda val: True))
+        attr.asdict(i, filter=attr.filters.exclude(lambda val: True))
+
 
 @pytest.fixture(
     name="member_validator",
@@ -569,15 +496,15 @@ class TestDeepIterable:
         v(None, a, [42])
 
     @pytest.mark.parametrize(
-        "member_validator, iterable_validator",
-        (
+        ("member_validator", "iterable_validator"),
+        [
             (instance_of(int), 42),
             (42, instance_of(list)),
             (42, 42),
             (42, None),
             ([instance_of(int), 42], 42),
             ([42, instance_of(int)], 42),
-        ),
+        ],
     )
     def test_noncallable_validators(
         self, member_validator, iterable_validator
@@ -588,8 +515,8 @@ class TestDeepIterable:
         with pytest.raises(TypeError) as e:
             deep_iterable(member_validator, iterable_validator)
         value = 42
-        message = "must be callable (got {value} that is a {type_}).".format(
-            value=value, type_=value.__class__
+        message = (
+            f"must be callable (got {value} that is a {value.__class__})."
         )
 
         assert message in e.value.args[0]
@@ -637,8 +564,8 @@ class TestDeepIterable:
         member_repr = "<instance_of validator for type <class 'int'>>"
         v = deep_iterable(member_validator)
         expected_repr = (
-            "<deep_iterable validator for iterables of {member_repr}>"
-        ).format(member_repr=member_repr)
+            f"<deep_iterable validator for iterables of {member_repr}>"
+        )
         assert expected_repr == repr(v)
 
     def test_repr_member_only_sequence(self):
@@ -649,13 +576,13 @@ class TestDeepIterable:
         """
         member_validator = [always_pass, instance_of(int)]
         member_repr = (
-            "_AndValidator(_validators=({func}, "
+            f"_AndValidator(_validators=({always_pass!r}, "
             "<instance_of validator for type <class 'int'>>))"
-        ).format(func=repr(always_pass))
+        )
         v = deep_iterable(member_validator)
         expected_repr = (
-            "<deep_iterable validator for iterables of {member_repr}>"
-        ).format(member_repr=member_repr)
+            f"<deep_iterable validator for iterables of {member_repr}>"
+        )
         assert expected_repr == repr(v)
 
     def test_repr_member_and_iterable(self):
@@ -670,8 +597,8 @@ class TestDeepIterable:
         v = deep_iterable(member_validator, iterable_validator)
         expected_repr = (
             "<deep_iterable validator for"
-            " {iterable_repr} iterables of {member_repr}>"
-        ).format(iterable_repr=iterable_repr, member_repr=member_repr)
+            f" {iterable_repr} iterables of {member_repr}>"
+        )
         assert expected_repr == repr(v)
 
     def test_repr_sequence_member_and_iterable(self):
@@ -682,16 +609,16 @@ class TestDeepIterable:
         """
         member_validator = [always_pass, instance_of(int)]
         member_repr = (
-            "_AndValidator(_validators=({func}, "
+            f"_AndValidator(_validators=({always_pass!r}, "
             "<instance_of validator for type <class 'int'>>))"
-        ).format(func=repr(always_pass))
+        )
         iterable_validator = instance_of(list)
         iterable_repr = "<instance_of validator for type <class 'list'>>"
         v = deep_iterable(member_validator, iterable_validator)
         expected_repr = (
             "<deep_iterable validator for"
-            " {iterable_repr} iterables of {member_repr}>"
-        ).format(iterable_repr=iterable_repr, member_repr=member_repr)
+            f" {iterable_repr} iterables of {member_repr}>"
+        )
 
         assert expected_repr == repr(v)
 
@@ -718,14 +645,14 @@ class TestDeepMapping:
         v(None, a, {"a": 6, "b": 7})
 
     @pytest.mark.parametrize(
-        "key_validator, value_validator, mapping_validator",
-        (
+        ("key_validator", "value_validator", "mapping_validator"),
+        [
             (42, instance_of(int), None),
             (instance_of(str), 42, None),
             (instance_of(str), instance_of(int), 42),
             (42, 42, None),
             (42, 42, 42),
-        ),
+        ],
     )
     def test_noncallable_validators(
         self, key_validator, value_validator, mapping_validator
@@ -737,8 +664,8 @@ class TestDeepMapping:
             deep_mapping(key_validator, value_validator, mapping_validator)
 
         value = 42
-        message = "must be callable (got {value} that is a {type_}).".format(
-            value=value, type_=value.__class__
+        message = (
+            f"must be callable (got {value} that is a {value.__class__})."
         )
 
         assert message in e.value.args[0]
@@ -791,8 +718,8 @@ class TestDeepMapping:
         v = deep_mapping(key_validator, value_validator)
         expected_repr = (
             "<deep_mapping validator for objects mapping "
-            "{key_repr} to {value_repr}>"
-        ).format(key_repr=key_repr, value_repr=value_repr)
+            f"{key_repr} to {value_repr}>"
+        )
         assert expected_repr == repr(v)
 
 
@@ -892,7 +819,7 @@ class TestLtLeGeGt:
         assert fields(Tester).value.validator.bound == self.BOUND
 
     @pytest.mark.parametrize(
-        "v, value",
+        ("v", "value"),
         [
             (lt, 3),
             (le, 3),
@@ -912,7 +839,7 @@ class TestLtLeGeGt:
         Tester(value)  # shouldn't raise exceptions
 
     @pytest.mark.parametrize(
-        "v, value",
+        ("v", "value"),
         [
             (lt, 4),
             (le, 5),
@@ -936,9 +863,7 @@ class TestLtLeGeGt:
         __repr__ is meaningful.
         """
         nv = v(23)
-        assert repr(nv) == "<Validator for x {op} {bound}>".format(
-            op=nv.compare_op, bound=23
-        )
+        assert repr(nv) == f"<Validator for x {nv.compare_op} {23}>"
 
 
 class TestMaxLen:
@@ -1150,13 +1075,8 @@ class TestNot_:
         v = not_(wrapped)
 
         assert (
-            (
-                "<not_ validator wrapping {wrapped!r}, "
-                "capturing {exc_types!r}>"
-            ).format(
-                wrapped=wrapped,
-                exc_types=v.exc_types,
-            )
+            f"<not_ validator wrapping {wrapped!r}, "
+            f"capturing {v.exc_types!r}>"
         ) == repr(v)
 
     def test_success_because_fails(self):
@@ -1190,8 +1110,8 @@ class TestNot_:
 
         assert (
             (
-                "not_ validator child '{!r}' did not raise a captured error"
-            ).format(always_passes),
+                f"not_ validator child '{always_passes!r}' did not raise a captured error"
+            ),
             a,
             always_passes,
             input_value,
@@ -1254,8 +1174,8 @@ class TestNot_:
 
         assert (
             (
-                "not_ validator child '{!r}' did not raise a captured error"
-            ).format(wrapped),
+                f"not_ validator child '{wrapped!r}' did not raise a captured error"
+            ),
             a,
             wrapped,
             input_value,
@@ -1286,8 +1206,8 @@ class TestNot_:
 
         assert (
             (
-                "not_ validator child '{!r}' did not raise a captured error"
-            ).format(instance_of((int, float))),
+                f"not_ validator child '{instance_of((int, float))!r}' did not raise a captured error"
+            ),
             a,
             wrapped,
             input_value,
@@ -1358,3 +1278,38 @@ class TestNot_:
             "'exc_types' must be a subclass of <class 'Exception'> "
             "(got <class 'str'>)."
         ) == e.value.args[0]
+
+
+class TestOr:
+    def test_in_all(self):
+        """
+        Verify that this validator is in ``__all__``.
+        """
+        assert or_.__name__ in validator_module.__all__
+
+    def test_success(self):
+        """
+        Succeeds if at least one of wrapped validators succeed.
+        """
+        v = or_(instance_of(str), always_pass)
+
+        v(None, simple_attr("test"), 42)
+
+    def test_fail(self):
+        """
+        Fails if all wrapped validators fail.
+        """
+        v = or_(instance_of(str), always_fail)
+
+        with pytest.raises(ValueError):
+            v(None, simple_attr("test"), 42)
+
+    def test_repr(self):
+        """
+        Returned validator has a useful `__repr__`.
+        """
+        v = or_(instance_of(int), instance_of(str))
+        assert (
+            "<or validator wrapping (<instance_of validator for type "
+            "<class 'int'>>, <instance_of validator for type <class 'str'>>)>"
+        ) == repr(v)
