@@ -13,7 +13,7 @@ def simple_class(
     eq=False,
     order=False,
     repr=False,
-    hash=False,
+    unsafe_hash=False,
     str=False,
     slots=False,
     frozen=False,
@@ -28,7 +28,7 @@ def simple_class(
         eq=eq or order,
         order=order,
         repr=repr,
-        hash=hash,
+        unsafe_hash=unsafe_hash,
         init=True,
         slots=slots,
         str=str,
